

clear all; close all; clc;  
for song = 4:10;
    disp(sprintf('BEGINNING AUDIO FILE %i...', song));
    wav_file = sprintf('../%i.wav',song);  % input audio filename
    song_fpath = sprintf('file_%i', song);
    mkdir(song_fpath);
    [ wholeFile, fs, nbits ] = wavread( wav_file );
    nchunks = length(wholeFile)/(fs*10)
    for chunk = 1:nchunks
        close all;
        disp(sprintf('Beginning chunk %i...', chunk));
        fpath = sprintf('chunk%i', chunk);
        mkdir(sprintf('%s/%s', song_fpath, fpath));

        % Define variables
        chop_rate = 0.95;
        Tw = 25;                % analysis frame duration (ms)
        Ts = 10;                % analysis frame shift (ms)
        alpha = 0.97;           % preemphasis coefficient
        M = 12;                 % number of filterbank channels 
        C = 12;                 % number of cepstral coefficients
        L = 22;                 % cepstral sine lifter parameter
        LF = 800;               % lower frequency limit (Hz)
        HF = 7000;              % upper frequency limit (Hz)

        npeaks = 20;
        minpeak = 0.1;

        bProgress = 0;
        % Read speech samples, sampling rate and precision from file
        [ wholeFile, fs, nbits ] = wavread( wav_file );

        %bounce down to mono
        if size(wholeFile, 2) > 1
            wholeFile = wholeFile(:,1) + wholeFile(:,2);
        end

        %truncate file
        wholeFile = wholeFile((chunk-1)*10*fs + 1:(chunk)*10*fs+1);
        sample_chop = ceil(Ts*0.001*fs*80);
        iterations = 10;
        x = wholeFile;
        for iter = 1:iterations
            %% step one: get MFCCs
            iter_fpath = sprintf('iteration_%i', iter);
            mkdir(sprintf('%s/%s/%s', song_fpath, fpath, iter_fpath));
            wavwrite(x,fs, sprintf('%s/%s/%s/iteration-%i.wav',song_fpath, fpath, iter_fpath, iter));
            disp(' generating MFCCs...');
            [ MFCCs, FBEs, frames ] = ...
                            mfcc( wholeFile, fs, Tw, Ts, alpha, @hamming, [LF HF], M, C+1, L );


            % Generate data needed for plotting 
            [ Nw, NF ] = size( frames );                % frame length and number of frames
            time_frames = [0:NF-1]*Ts*0.001+0.5*Nw/fs;  % time vector (s) for frames 
            time = [ 0:length(wholeFile)-1 ]/fs;           % time vector (s) for signal samples 
            logFBEs = 20*log10( FBEs );                 % compute log FBEs for plotting
            logFBEs_floor = max(logFBEs(:))-50;         % get logFBE floor 50 dB below max
            logFBEs( logFBEs<logFBEs_floor ) = logFBEs_floor; % limit logFBE dynamic range


            % Generate plots
            figure('Position', [30 30 800 600], 'PaperPositionMode', 'auto', ... 
                      'color', 'w', 'PaperOrientation', 'landscape', 'Visible', 'on' ); 

            subplot( 211 );
            plot( time, wholeFile, 'k' );
            xlim( [ min(time_frames) max(time_frames) ] );
            xlabel( 'Time (s)' ); 
            ylabel( 'Amplitude' ); 
            title( sprintf('Original Field Recording waveform: Iteration %i', iter)); 

            subplot( 212 );
            imagesc( time_frames, [1:C], MFCCs(2:end,:) ); % HTK's TARGETKIND: MFCC
            %imagesc( time_frames, [1:C+1], MFCCs );       % HTK's TARGETKIND: MFCC_0
            axis( 'xy' );
            xlim( [ min(time_frames) max(time_frames) ] );
            xlabel( 'Time (s)' ); 
            ylabel( 'Cepstrum index' );
            title( 'Mel frequency cepstrum' );

            % Set color map to grayscale
            %colormap = 1- colormap('gray'); 

            % Print figure to pdf and png files
            print('-dpdf', sprintf('%s/%s/%s/%s-iteration%i.pdf',song_fpath, fpath, iter_fpath, 'mfccs', iter)); 
            print('-dpng', sprintf('%s/%s/%s/%s-iteration%i.png', song_fpath, fpath, iter_fpath, 'mfccs', iter)); 

            %truncate mfccs
            MFCCs = MFCCs(2:end-1, :);
            g = zeros(length(MFCCs));
            figure;
            colormap gray
            %% actual stuff
            thresh = length(MFCCs(:,1)) * 0.4; %mean(mean(abs(MFCCs))) + 39*std(std(abs(MFCCs)));
            disp('  generating covariance matrix...')
            for  i = 1:length(MFCCs)
                for j = 1:length(MFCCs)
                    %normalize MFCCs by frame
                    MFCCs(:, i) = MFCCs(:,i)/max(MFCCs(:,i));
                    MFCCs(:, j) = MFCCs(:,j)/max(MFCCs(:,j));
                    if (sum(abs(MFCCs(:,i))) > thresh) || (sum(abs(MFCCs(:,j))) > thresh)
                        g(i,j) = 0.5*(1+(dot(abs(MFCCs(:,i)), abs(MFCCs(:,j)))/(norm(abs(MFCCs(:,i)) * abs(MFCCs(:,j))')))^2);
                    end
                end
            end
            imagesc(g);
            title(sprintf('Covariance, iteration %i', iter));
            % Print figure to pdf and png files
            % Print figure to pdf and png files
            print('-dpdf', sprintf('%s/%s/%s/%s-iteration%i.pdf', song_fpath, fpath, iter_fpath, 'covariances', iter)); 
            print('-dpng', sprintf('%s/%s/%s/%s-iteration%i.png', song_fpath, fpath, iter_fpath, 'covariances', iter)); 
            disp('generating hough transform...')
            [H,T,R] = hough(g, 'Theta', -45);
            figure;
            plot(H);
            title(sprintf('Hough Tranform: iteration %i', iter))
            print('-dpdf', sprintf('%s/%s/%s/%s-iteration%i.pdf', song_fpath, fpath, iter_fpath, 'hough', iter)); 
            print('-dpng', sprintf('%s/%s/%s/%s-iteration%i.png', song_fpath, fpath, iter_fpath, 'hough', iter)); 

            %% get peak locations
            %set diag to 0
            g = g - diag(diag(g));
        %     loc = zeros(size(g));
            %[z m p l] = extrema2(g);
            disp('   finding similar sounds...')
            try
             for p = 1:length(g)
                 [pks(:,p), a] = findpeaks(g(:,p), 'NPEAKS', npeaks, 'MINPEAKHEIGHT', minpeak);
                 locs(:,p) = a;
             end
             pk_vec = zeros(1,length(locs(:,p)));
             for p = 1:length(locs(:,1))
                 for k = 1:length(locs)
                     if p+k < length(locs)
                        pk_vec(p) = pk_vec(p) + pks(p,p+k);
                     end
                 end
             end

             for p = 1:5
                 [pk,i] = max(pk_vec);
                 loc_vec(p) = locs(i,1);
                 pk_vec(i) = 0;
             end

             locs_in_samples = loc_vec * Ts * fs * 0.001;
            %% stitch together file thing from our biggest similarities
            disp('   truncating audio...');
            x = wholeFile(locs_in_samples(1):locs_in_samples(1)+sample_chop);
            for n = 2:length(loc_vec)/2
                x = [x',wholeFile(locs_in_samples(n*2):locs_in_samples(n*2))']';
            end
            wholeFile = x;
            sample_chop = sample_chop * chop_rate;
            Tw = ceil(Tw * chop_rate);
            Ts = ceil(Ts * chop_rate);
            bProgress = 1;
            catch
               if bProgress == 0
                    minpeak = minpeak - 0.04;
               else
                   disp(sprintf('            chunk complete. number of iterations: %i', iter));
                   break
               end
            end
        end
    end
end